@@ -3,7 +3,7 @@ package DateTime::Format::Strptime;
 use strict;
 use warnings;
 
-our $VERSION = '1.62';
+our $VERSION = '1.65';
 
 use Carp qw( carp croak );
 use DateTime 1.00;
@@ -24,7 +24,7 @@ use constant PERL_58 => $] < 5.010;
 
 {
     my $spec = {
-        pattern => { type => SCALAR },
+        pattern   => { type => SCALAR },
         time_zone => {
             type     => SCALAR | OBJECT,
             optional => 1,
@@ -191,6 +191,11 @@ use constant PERL_58 => $] < 5.010;
         'UTC'    => '+0000',
     );
 
+    for my $i ( map { sprintf( '%02d', $_ ) } 1 .. 12 ) {
+        $zone_map{ '-' . $i } = '-' . $i . '00';
+        $zone_map{ '+' . $i } = '+' . $i . '00';
+    }
+
     sub _build_zone_map {
         return \%zone_map;
     }
@@ -231,12 +236,16 @@ sub parse_datetime {
     # We need to copy the %args here because _munge_args will delete keys in
     # order to turn this into something that can be passed to a DateTime
     # constructor.
-    my ( $constructor, $args ) = $self->_munge_args( {%args} )
-        or return;
+    my ( $constructor, $args, $post_construct )
+        = $self->_munge_args( {%args} );
+    return unless $constructor && $args;
 
     my $dt = try { DateTime->$constructor($args) };
     $self->_our_croak('Parsed values did not produce a valid date')
         unless $dt;
+    if ($post_construct) {
+        $post_construct->($dt);
+    }
     return unless $dt && $self->_check_dt( $dt, \%args );
 
     $dt->set_time_zone( $self->{time_zone} )
@@ -274,7 +283,6 @@ sub _build_parser {
 
     my $regex;
     my @fields;
-    my @postprocess;
 
     while (
         $pattern =~ /
@@ -317,7 +325,7 @@ sub _build_parser {
     }
 
     return {
-        regex  => qr/\A\s*$regex\s*\z/,
+        regex  => qr/\A\s*$regex/,
         fields => \@fields,
     };
 }
@@ -412,7 +420,7 @@ sub _build_parser {
             field => 'time_zone_offset',
         },
         Z => {
-            regex => qr/[a-zA-Z]{1,6}/,
+            regex => qr/[a-zA-Z]{1,6}|[\-\+][0-9]{2}/,
             field => 'time_zone_abbreviation',
         },
     );
@@ -640,24 +648,39 @@ sub _token_re_for {
             $args->{$k} =~ s/^\s+//;
         }
 
-        # If we parsed "12345" we treat it as "123450000" but if we parsed
-        # "000123456" we treat it as 123,456 nanoseconds. This is all a bit
-        # weird and confusing but it matches how this module has always
-        # worked.
-        $args->{nanosecond} *= 10**( 9 - length $args->{nanosecond} )
-            if defined $args->{nanosecond} && length $args->{nanosecond} != 9;
+        if ( defined $args->{nanosecond} ) {
+
+            # If we parsed "12345" we treat it as "123450000" but if we parsed
+            # "000123456" we treat it as 123,456 nanoseconds. This is all a bit
+            # weird and confusing but it matches how this module has always
+            # worked.
+            $args->{nanosecond} *= 10**( 9 - length $args->{nanosecond} )
+                if length $args->{nanosecond} != 9;
+
+            # If we parsed 000000123 we want to turn this into a number.
+            $args->{nanosecond} += 0;
+        }
 
         for my $k (qw( year month day )) {
             $args->{$k} = 1 unless defined $args->{$k};
         }
 
         if ( defined $args->{epoch} ) {
-            $args->{epoch} .= q{.} . $args->{nanosecond}
-                if defined $args->{nanosecond};
+
+            # We don't want to pass a non-integer epoch value since that gets
+            # truncated as of DateTime 1.22. Instead, we'll set the nanosecond
+            # to parsed value after constructing the object. This is a hack,
+            # but it's the best I can come up with.
+            my $post_construct;
+            if ( my $nano = $args->{nanosecond} ) {
+                $post_construct = sub { $_[0]->set( nanosecond => $nano ) };
+            }
+
             delete @{$args}{
                 qw( day_of_year year month day hour minute second nanosecond )
             };
-            return ( 'from_epoch', $args );
+
+            return ( 'from_epoch', $args, $post_construct );
         }
         elsif ( $args->{day_of_year} ) {
             delete @{$args}{qw( epoch month day )};
@@ -1008,7 +1031,7 @@ DateTime::Format::Strptime - Parse and format strp and strf time patterns
 
 =head1 VERSION
 
-version 1.62
+version 1.65
 
 =head1 SYNOPSIS
 
@@ -1052,6 +1075,8 @@ reverse of C<strftime(3)>, for C<DateTime>. While C<strftime> takes a
 C<DateTime> and a pattern and returns a string, C<strptime> takes a string and
 a pattern and returns the C<DateTime> object associated.
 
+=encoding UTF-8
+
 =for Pod::Coverage parse_duration format_duration
 
 =head1 METHODS
@@ -1336,9 +1361,15 @@ Rick Measham <rickm@cpan.org>
 
 =back
 
+=head1 CONTRIBUTOR
+
+=for stopwords D. Ilmari Mannsåker
+
+D. Ilmari Mannsåker <ilmari.mannsaker@net-a-porter.com>
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2015 by Dave Rolsky.
+This software is Copyright (c) 2016 by Dave Rolsky.
 
 This is free software, licensed under:
 
